/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.agent.management.api.rest;

import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

/**
 * JOnAS REST Management API
 * @author Jeremy Cazaux
 */
@Path("/")
public interface IManagement {

    /**
     * @response.representation.200.qname {http://localhost:9000/jonas-api/}task
     * @response.representation.200.mediaType application/xml
     *
     * @return the list of tasks
     */
    @GET
    @Path("/task")
    @Produces(MediaType.APPLICATION_XML)
    Response getTasks();

    /**
     * @param taskId The id of a task
     *
     * @response.representation.200.qname {http://localhost:9000/jonas-api/}task/{taskid}
     * @response.representation.200.mediaType application/xml
     *
     * @return the associated task
     */
    @GET
    @Path("/task/{taskid}")
    @Produces(MediaType.APPLICATION_XML)
    Response getTask(@PathParam("taskid") final Long taskId);

    /**
     * @param taskId Id of the task to cancel
     *
     * @response.representation.200.qname {http://localhost:9000/jonas-api/}task/{taskid}/action/cancel
     */
    @POST
    @Path("/task/{taskid}/action/cancel")
    Response cancelTask(@PathParam("taskid") final Long taskId);

    /**
     * Create a new JOnAS instance
     * @param topology Topology file
     * @param serverName The server name
     *
     * @response.representation.202.qname {http://localhost:9000/jonas-api/}server/{servername}
     * @response.representation.202.mediaType application/xml
     *
     * @return a {@link org.ow2.jonas.agent.management.api.xml.Task}
     */
    @PUT
    @Path("/server/{servername}")
    @Consumes(MediaType.APPLICATION_XML)
    @Produces(MediaType.APPLICATION_XML)
    Response createJOnASInstance(final String topology, @PathParam("servername") final String serverName);

    /**
     * Start a new JOnAS instance
     * @param serverName The name of the server to start
     *
     * @response.representation.202.qname {http://localhost:9000/jonas-api/}server/{servername}/action/start
     * @response.representation.202.mediaType application/xml
     *
     * @return a {@link org.ow2.jonas.agent.management.api.xml.Task}
     */
    @POST
    @Path("/server/{servername}/action/start")
    @Produces(MediaType.APPLICATION_XML)
    Response startJOnASInstance(@PathParam("servername") final String serverName);

    /**
     * Stop a JOnAS instance
     * @param serverName The name of the server to stop
     *
     * @response.representation.202.qname {http://localhost:9000/jonas-api/}server/{servername}/action/stop
     * @response.representation.202.mediaType application/xml
     *
     * @return a {@link org.ow2.jonas.agent.management.api.xml.Task}
     */
    @POST
    @Path("/server/{servername}/action/stop")
    @Produces(MediaType.APPLICATION_XML)
    Response stopJOnASInstance(@PathParam("servername") final String serverName);

    /**
     * @response.representation.200.qname {http://localhost:9000/jonas-api/}server
     * @response.representation.200.mediaType application/xml
     *
     * @return reports for each JOnAS instances
     */
    @GET
    @Path("/server")
    @Produces(MediaType.APPLICATION_XML)
    Response getJOnASInstancesReport();

    /**
     * @param serverName A server name
     *
     * @response.representation.200.qname {http://localhost:9000/jonas-api/}server/{servername}
     * @response.representation.200.mediaType application/xml
     *
     * @return the report of the server associated to the given server name
     */
    @GET
    @Path("/server/{servername}")
    @Produces(MediaType.APPLICATION_XML)
    Response getJOnASInstanceReport(@PathParam("servername") final String serverName);

    /**
     * Delete a JOnAS instance
     * @param serverName The name of the JOnAS instance to delete
     *
     * @response.representation.204.qname {http://localhost:9000/jonas-api/}server/{servername}
     */
    @DELETE
    @Path("/server/{servername}")
    Response deleteJOnASInstance(@PathParam("servername") final String serverName);

    /**
     * Stop all managed servers
     */
    void stopManagedServers();

    /**
     * Deploy an application on a JOnAS instance
     *
     * @param serverName The name of the server where to deploy the application
     *
     * @param applicationName The name of the application
     *
     * @response.representation.202.qname {http://localhost:9000/jonas-api/}server/
     *                                          {servername}/app/{appname}/action/deploy
     * @response.representation.202.mediaType application/xml
     *
     * @return a {@link org.ow2.jonas.agent.management.api.xml.Task}
     */
    @POST
    @Path("/server/{servername}/app/{appname}/action/deploy")
    @Produces(MediaType.APPLICATION_XML)
    @Consumes(MediaType.APPLICATION_OCTET_STREAM)
    Response deployApplication(byte[] application, @PathParam("servername") final String serverName,
                               @PathParam("appname") final String applicationName);

    /**
     * Undeploy an application on a JOnAS instance
     *
     * @param serverName The name of the server where to undeploy the application
     *
     * @param applicationName The name of the application
     *
     * @response.representation.202.qname {http://localhost:9000/jonas-api/}server/
     *                                          {servername}/app/{appname}/action/undeploy
     * @response.representation.202.mediaType application/xml
     *
     * @return a {@link org.ow2.jonas.agent.management.api.xml.Task}
     */
    @POST
    @Path("/server/{servername}/app/{appname}/action/undeploy")
    @Produces(MediaType.APPLICATION_XML)
    Response undeployApplication(@PathParam("servername") final String serverName,
                               @PathParam("appname") final String applicationName);


    /**
     * Get the applications list of a JOnAS instance
     *
     * @param serverName The name of the server
     *
     * @response.representation.202.qname {http://localhost:9000/jonas-api/}server/${servername}/app
     * @response.representation.202.mediaType application/xml
     *
     * @return a {@link org.ow2.jonas.agent.management.api.xml.AppList}
     */
    @GET
    @Path("/server/{servername}/app")
    @Produces(MediaType.APPLICATION_XML)
    Response getApplicationList(@PathParam("servername") final String serverName);


    /**
     * Get information about an application on a JOnAS instance
     *
     * @param serverName The name of the server
     *
     * @param applicationName The name of the application
     *
     * @response.representation.202.qname {http://localhost:9000/jonas-api/}server/{servername}/app/{appname}
     * @response.representation.202.mediaType application/xml
     *
     * @return a {@link org.ow2.jonas.agent.management.api.xml.App}
     */
    @GET
    @Path("/server/{servername}/app/{appname}")
    @Produces(MediaType.APPLICATION_XML)
    Response getApplication(@PathParam("servername") final String serverName,
                               @PathParam("appname") final String applicationName);

}

